/* -*- mode:C; c-file-style:"linux"; tab-width:8; -*- */

#include <string.h>
#include <math.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libecal/e-cal-time-util.h>
#include <libical/icaltime.h>
#include <gconf/gconf-client.h>

#ifdef WITH_HILDON
#include <gdk/gdkkeysyms.h>
#include <libosso.h>
#include <hildon-widgets/hildon-window.h>
#include <hildon-widgets/hildon-program.h>
#include <hildon-widgets/hildon-time-editor.h>
#include <hildon-widgets/hildon-date-editor.h>
#include <glade/glade-build.h>
#endif

#include "dates_view.h"
#include "gconf-bridge.h"

#define XML_FILE PKGDATADIR "/dates.glade"
#define CALENDAR_GCONF_PREFIX   "/apps/evolution/calendar"
#define CALENDAR_GCONF_SOURCES CALENDAR_GCONF_PREFIX "/sources"
#define TIME_MARKUP "%k:%M\n<small>%d %b %Y</small>"
#define DATES_GCONF_PREFIX	"/apps/dates"
#define DATES_GCONF_ZOOM	DATES_GCONF_PREFIX "/zoom"

#define CONVERT_CHILD_ITER(_filter,_child,_iter)\
	(gtk_tree_model_filter_convert_child_iter_to_iter (\
	GTK_TREE_MODEL_FILTER (_filter), _child, _iter))

enum {
	COL_CALNAME,
	COL_CALPTR,
	COL_NOTREADONLY,
	COL_LAST
};

enum {
	NONE = 0,
	NEW_EVENT,
	CAL_CHANGE
};

enum {
	DAY = 1,
	WEEK,
	FORTNIGHT,
	MONTH,
	YEAR,
	OTHER
};

typedef struct {
	DatesView *view;
	guint zoom;
	GladeXML *xml;
	ECalComponent *comp;
	ECal *cal;
	gboolean read_only;
	gboolean first_load;
	GtkListStore *cal_list_store;
	gchar *uri_uid;
	GList *widgets;
	gint waiting;
} DatesViewData;

static void dates_zoom_change (guint pos, DatesView *dates_view);
static void dates_update_calendars (ESourceList *cal_list, DatesViewData *d);

GtkWidget *
create_dates_view ()
{
	GtkWidget *widget;
	
	widget = dates_view_new ();
	/* TODO: This is temporary until the widget sizes itself properly */
	gtk_widget_set_size_request (widget, 240, 240);
	gtk_widget_show (widget);
	
	return widget;
}

/* From contacts src/contacts-utils.c */
static GList *
contacts_set_widget_desensitive_recurse (GtkWidget *widget, GList **widgets)
{
	if (GTK_IS_WIDGET (widget)) {
		if (GTK_WIDGET_SENSITIVE (widget) &&
		    GTK_WIDGET_VISIBLE (widget)) {
			gtk_widget_set_sensitive (widget, FALSE);
			*widgets = g_list_append (*widgets, widget);
		}
		
		if (GTK_IS_TABLE (widget) || GTK_IS_HBOX (widget) ||
		    GTK_IS_VBOX (widget) || GTK_IS_EVENT_BOX (widget)) {
			GList *c, *children = gtk_container_get_children (
				GTK_CONTAINER (widget));
			
			for (c = children; c; c = c->next) {
				contacts_set_widget_desensitive_recurse (
					c->data, widgets);
			}
			g_list_free (children);
		}
	}
	
	return *widgets;
}

GList *
contacts_set_widgets_desensitive (GtkWidget *widget)
{
	GList *list = NULL;
	
	contacts_set_widget_desensitive_recurse (widget, &list);
	
	return list;
}

void
contacts_set_widgets_sensitive (GList *widgets)
{
	GList *w;
	
	for (w = widgets; w; w = w->next) {
		gtk_widget_set_sensitive (GTK_WIDGET (w->data), TRUE);
	}
}

static void
dates_date_change (DatesViewData *data, int direction)
{
	icaltimetype *time, *oldtime;
	
	oldtime = (icaltimetype *)dates_view_get_date (data->view);
	time = g_new (icaltimetype, 1);
	g_memmove (time, oldtime, sizeof (icaltimetype));

	if (data->zoom == 16)
		time->year += direction;
	else if (data->zoom >= 11) {
		int days;
		time->month += direction;
		if (time->month > 12) {
			time->month = 1;
			time->year++;
		} else if (time->month < 1) {
			time->month = 12;
			time->year--;
		}
		days = icaltime_days_in_month (time->month, time->year);
		if (time->day > days) time->day = days;
	} else if (data->zoom >= 8)
		icaltime_adjust (time, direction * 7, 0, 0, 0);
	else
		icaltime_adjust (time, direction, 0, 0, 0);
	
	dates_view_set_date (data->view, time);
	
	g_free (time);
}

static void
dates_back_cb (GtkButton *button, DatesViewData *data)
{
	dates_date_change (data, -1);
}

static void
dates_forward_cb (GtkButton *button, DatesViewData *data)
{
	dates_date_change (data, 1);
}

static void
dates_today_cb (GtkButton *button, DatesViewData *data)
{
	icaltimetype today = icaltime_today ();
	dates_view_set_date (data->view, &today);
}
#if 0
static void
dates_day_cb (GtkToggleButton *button, DatesViewData *d)
{
	d->zoom = 3;	
	dates_zoom_change (
		3, DATES_VIEW (glade_xml_get_widget (d->xml, "dates_view")));
}

static void
dates_week_cb (GtkToggleButton *button, DatesViewData *d)
{
	d->zoom = 10;	
	dates_zoom_change (
		10, DATES_VIEW (glade_xml_get_widget (d->xml, "dates_view")));
}

static void
dates_month_cb (GtkToggleButton *button, DatesViewData *d)
{
	d->zoom = 11;	
	dates_zoom_change (
		11, DATES_VIEW (glade_xml_get_widget (d->xml, "dates_view")));
}

static void
dates_year_cb (GtkToggleButton *button, DatesViewData *d)
{
	d->zoom = 16;	
	dates_zoom_change (
		16, DATES_VIEW (glade_xml_get_widget (d->xml, "dates_view")));
}
#endif
static void
dates_zoom_in_cb (GtkButton *button, DatesViewData *data)
{
	switch (data->zoom) {
		case 6:
			gtk_widget_set_sensitive (glade_xml_get_widget (
				data->xml, "zoom_in_button"), FALSE);
			data->zoom = 3;
			break;
/*		case 9:
			data->zoom = 6;
			break;
		case 10:
			data->zoom = 9;
			break;*/
		case 11:
			data->zoom = 6;
			break;
		case 16:
			gtk_widget_set_sensitive (glade_xml_get_widget (
				data->xml, "zoom_out_button"), TRUE);
			data->zoom = 11;
			break;
		default:
			return;
	}
	
/*	if (data->zoom > 1) {
		data->zoom--;
		dates_zoom_change (data->zoom, data->view);
	}*/
	
	dates_zoom_change (data->zoom, data->view);
}

static void
dates_zoom_out_cb (GtkButton *button, DatesViewData *data)
{
	switch (data->zoom) {
		case 3:
			gtk_widget_set_sensitive (glade_xml_get_widget (
				data->xml, "zoom_in_button"), TRUE);
			data->zoom = 6;
			break;
		case 6:
			data->zoom = 11;
			break;
/*		case 9:
			data->zoom = 10;
			break;
		case 10:
			data->zoom = 11;
			break;*/
		case 11:
			gtk_widget_set_sensitive (glade_xml_get_widget (
				data->xml, "zoom_out_button"), FALSE);
			data->zoom = 16;
			break;
		default:
			return;
	}

/*	if (data->zoom < 16) {
		data->zoom++;
		dates_zoom_change (data->zoom, data->view);
	}*/

	dates_zoom_change (data->zoom, data->view);
}

static void
dates_zoom_change (guint pos, DatesView *dates_view)
{
	GConfClient *client;
	
	switch (pos) {
		case 1 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 1);
			dates_view_set_visible_hours (dates_view, 4);
			break;
		case 2 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 1);
			dates_view_set_visible_hours (dates_view, 8);
			break;
		case 3 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 1);
			dates_view_set_visible_hours (dates_view, 12);
			break;
		case 4 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 1);
			dates_view_set_visible_hours (dates_view, 12);
			break;
		case 5 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 2);
			dates_view_set_visible_hours (dates_view, 12);
			break;
		case 6 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 1);
			dates_view_set_visible_hours (dates_view, 24);
			break;
		case 7 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 4);
			dates_view_set_visible_hours (dates_view, 12);
			break;
		case 8 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 5);
			dates_view_set_visible_hours (dates_view, 12);
			break;
		case 9 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 7);
			dates_view_set_visible_hours (dates_view, 12);
			break;
		case 10 :
			dates_view_set_visible_months (dates_view, 0);
			dates_view_set_visible_days (dates_view, 7);
			dates_view_set_visible_hours (dates_view, 24);
			break;
		case 11 :
			dates_view_set_visible_months (dates_view, 1);
			dates_view_set_visible_days (dates_view, 7);
			dates_view_set_visible_hours (dates_view, 24);
			break;
		case 12 :
			dates_view_set_visible_months (dates_view, 2);
			dates_view_set_visible_days (dates_view, 7);
			dates_view_set_visible_hours (dates_view, 24);
			break;
		case 13 :
			dates_view_set_visible_months (dates_view, 3);
			dates_view_set_visible_days (dates_view, 7);
			dates_view_set_visible_hours (dates_view, 24);
			break;
		case 14 :
			dates_view_set_visible_months (dates_view, 4);
			dates_view_set_visible_days (dates_view, 7);
			dates_view_set_visible_hours (dates_view, 24);
			break;
		case 15 :
			dates_view_set_visible_months (dates_view, 8);
			dates_view_set_visible_days (dates_view, 7);
			dates_view_set_visible_hours (dates_view, 24);
			break;
		case 16 :
			dates_view_set_visible_months (dates_view, 12);
			dates_view_set_visible_days (dates_view, 7);
			dates_view_set_visible_hours (dates_view, 24);
			break;
	}
	
	client = gconf_client_get_default ();
	gconf_client_set_int (client, DATES_GCONF_ZOOM, pos, NULL);
}

static void
dates_date_changed_cb (DatesView *view, DatesViewData *d)
{
	struct tm timem;
	gchar buffer[256];
	const icaltimetype *date = dates_view_get_date (view);
#ifndef WITH_HILDON
	gchar *text;
#endif
	
	timem = icaltimetype_to_tm ((icaltimetype *)date);
	strftime (buffer, 255, "%a, %F", &timem);

#ifdef WITH_HILDON
	gtk_window_set_title (GTK_WINDOW (
		glade_xml_get_widget (d->xml, "main_window")), buffer);
#else
	text = g_strdup_printf ("%s - %s", buffer, _("Dates"));
	gtk_window_set_title (GTK_WINDOW (
		glade_xml_get_widget (d->xml, "main_window")), text);
	g_free (text);
#endif

	strftime (buffer, 255, "%A %d %B, %Y", &timem);
	gtk_label_set_text (GTK_LABEL (
		glade_xml_get_widget (d->xml, "header_label")), buffer);
}


static void
dates_about_cb (GtkWidget *widget, DatesViewData *d)
{
	GdkPixbuf *pixb;
	GError *error = NULL;

	/* NOTE: The hildon version of the about menu does not handle the
	 * credits button well (it is misplaced and not thematised), so we add
	 * the authors to the (c) string.
	 */
	#ifndef WITH_HILDON
	gchar *authors[] = {
		"Principal author: Chris Lord <chris@o-hand.com>",
		"Maemo port:       Tomas Frydrych <tf@o-hand.com>",
		NULL
	};
	#endif

	pixb = gdk_pixbuf_new_from_file (
		PKGDATADIR G_DIR_SEPARATOR_S "oh-about-logo.png", &error);  

	if (!pixb) {
		g_assert (error);
		g_warning (error->message);
	}

	gtk_show_about_dialog (
		GTK_WINDOW (glade_xml_get_widget (d->xml, "main_window")),
		#ifndef WITH_HILDON
			"copyright",
			"(c) 2006 OpenedHand Ltd",

			"authors",
			authors,
		#else
			"copyright",
			"(c) 2006 OpenedHand Ltd\n"
			"Authors: Chris Lord <chris@o-hand.com>, "
			"Tomas Frydrych <tf@o-hand.com>",
		#endif

		"logo",
		pixb,

		"name",
		PACKAGE_NAME,

		"version",
		PACKAGE_VERSION,

		"website",
		"http://www.openedhand.com",

		NULL);
}

static void
dates_fill_details_dialog (DatesView *view, DatesViewData *d)
{
	const gchar *location;
	GSList *text_list;
	ECalComponentDateTime time;
	struct tm timem;
	gchar time_text[256];
	GtkTextBuffer *buffer;
	ECalComponentText summary;
	GtkWidget *widget;

	/* Set summary entry */
	e_cal_component_get_summary (d->comp, &summary);
	widget = glade_xml_get_widget (d->xml, "details_summary_entry");
	if (summary.value)
		gtk_entry_set_text (GTK_ENTRY (widget), summary.value);
	else if (summary.altrep)
		gtk_entry_set_text (GTK_ENTRY (widget), summary.altrep);
	else
		gtk_entry_set_text (GTK_ENTRY (widget), "");
	
	/* Set location entry */
	e_cal_component_get_location (d->comp, &location);
	if (location)
		gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget (
			d->xml, "details_location_entry")), location);
	else
		gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget (
			d->xml, "details_location_entry")), "");
	
	/* Set event description textview */
	/* Note: Docs say that only journal entries can have more than one
	 * description, so just use text_list->data
	 */
	e_cal_component_get_description_list (d->comp, &text_list);
	buffer = gtk_text_view_get_buffer (
		GTK_TEXT_VIEW (glade_xml_get_widget (
			d->xml, "details_textview")));
	gtk_text_buffer_set_text (buffer, "", -1);
	if (text_list) {
		ECalComponentText *desc = text_list->data;
		if (desc->value)
			gtk_text_buffer_set_text (buffer, desc->value, -1);
		e_cal_component_free_text_list (text_list);
	}
	
	/* Set time buttons and dialog */
	e_cal_component_get_dtstart (d->comp, &time);
	timem = icaltimetype_to_tm (time.value);
	strftime (time_text, 256, TIME_MARKUP, &timem);
	gtk_label_set_markup (GTK_LABEL (
		glade_xml_get_widget (d->xml, "details_start_label")),
		time_text);
	e_cal_component_free_datetime (&time);

	e_cal_component_get_dtend (d->comp, &time);
	timem = icaltimetype_to_tm (time.value);
	strftime (time_text, 256, TIME_MARKUP, &timem);
	gtk_label_set_markup (GTK_LABEL (
		glade_xml_get_widget (d->xml, "details_end_label")),
		time_text);
	e_cal_component_free_datetime (&time);
}

static void
dates_event_selected_cb (DatesView *view, DatesViewData *d)
{
	GtkWidget *widget;

	if (d->comp) g_object_unref (d->comp);
	d->comp = dates_view_get_selected_event (d->view);

	if (d->comp) {
		GtkTreeIter iter;
		gboolean read_only = TRUE, cal_found = FALSE;
		GtkTreeModel *model;

		g_debug ("Selected event RID: %s",
			e_cal_component_get_recurid_as_string (d->comp));

		d->cal = dates_view_get_selected_event_cal (d->view);
			
/*		widget = glade_xml_get_widget (d->xml, "header_label");
		gtk_label_set_markup (GTK_LABEL (widget),
			"<big><b>Enter details</b></big>");*/
		
		/* Set the details calendar combo-box */
		widget = glade_xml_get_widget (
			d->xml, "details_calendar_combobox");
		model = GTK_TREE_MODEL (d->cal_list_store);
		if (gtk_tree_model_get_iter_first (model, &iter)) {
			do {
				ECal *c;

				gtk_tree_model_get (
					model, &iter, COL_CALPTR, &c, -1);

				if (c != d->cal) continue;
				cal_found = TRUE;
				
				gtk_tree_model_get (model, &iter,
					COL_NOTREADONLY, &read_only,
					-1);
				read_only = !read_only;
				d->read_only = read_only;
				if (!read_only) {
					GtkTreeIter filter_iter;
					
					model = gtk_combo_box_get_model (
						GTK_COMBO_BOX (widget));
					CONVERT_CHILD_ITER (model, &filter_iter,
						&iter);
					
					gtk_combo_box_set_active_iter (
						GTK_COMBO_BOX (widget),
						&filter_iter);
				} else {
					g_debug ("Read-only event selected");
				}
				break;
			} while (gtk_tree_model_iter_next (model, &iter));
			if (!cal_found)
				g_warning ("Event selected with "
					"unknown calendar");
		} else {
			g_warning ("Error accessing calendar list");
		}
				
		widget = glade_xml_get_widget (d->xml, "open_button");
		gtk_widget_set_sensitive (widget, !read_only);
		widget = glade_xml_get_widget (
			d->xml, "calendar_open_menuitem");
		gtk_widget_set_sensitive (widget, !read_only);
		
		widget = glade_xml_get_widget (d->xml, "delete_button");
		gtk_widget_set_sensitive (widget, !read_only);
		widget = glade_xml_get_widget (
			d->xml, "calendar_delete_menuitem");
		gtk_widget_set_sensitive (widget, !read_only);
	} else {
		d->cal = NULL;
		if (!d->waiting)
			gtk_widget_hide (glade_xml_get_widget (
				d->xml, "details_dialog"));

		widget = glade_xml_get_widget (d->xml, "open_button");
		gtk_widget_set_sensitive (widget, FALSE);
		widget = glade_xml_get_widget (
			d->xml, "calendar_open_menuitem");
		gtk_widget_set_sensitive (widget, FALSE);

		widget = glade_xml_get_widget (d->xml, "delete_button");
		gtk_widget_set_sensitive (widget, FALSE);
		widget = glade_xml_get_widget (
			d->xml, "calendar_delete_menuitem");
		gtk_widget_set_sensitive (widget, FALSE);
	}
}

static void
dates_commit_event_cb (GtkWidget *source, DatesViewData *data,
		       CalObjModType type)
{
	/* Commit changes */
	g_debug ("Committing event (%p) changes", data->comp);
	e_cal_component_commit_sequence (data->comp);
	e_cal_modify_object (data->cal,
		e_cal_component_get_icalcomponent (data->comp),
		type, NULL);
}

static void
dates_event_moved_cb (DatesView *view, ECalComponent *comp, DatesViewData *d)
{
	ECalComponentDateTime start, end;
	struct icaldurationtype duration = icaldurationtype_null_duration ();

	if (d->comp) g_object_unref (d->comp);
	d->comp = g_object_ref (comp);

	e_cal_component_get_dtstart (d->comp, &start);
	if (!start.value->is_date) {
		e_cal_component_get_dtend (d->comp, &end);
		duration = icaltime_subtract (*end.value, *start.value);
		*end.value = icaltime_add (*start.value, duration);
		e_cal_component_set_dtstart (d->comp, &start);
		e_cal_component_set_dtend (d->comp, &end);
		e_cal_component_free_datetime (&end);
	}
	e_cal_component_free_datetime (&start);
	dates_commit_event_cb (NULL, d, CALOBJ_MOD_THIS);
}

static void
dates_event_sized_cb (DatesView *view, ECalComponent *comp, DatesViewData *d)
{
	ECalComponentDateTime end;

	if (d->comp) g_object_unref (d->comp);
	d->comp = g_object_ref (comp);

	e_cal_component_get_dtend (d->comp, &end);
	e_cal_component_set_dtend (d->comp, &end);
	e_cal_component_free_datetime (&end);
	dates_commit_event_cb (NULL, d, CALOBJ_MOD_THIS);
}

static void
dates_details_time_entry_changed (GtkEditable *entry, gchar *new_text,
				  gint new_text_length,
				  gint *position,
				  DatesViewData *d)
{
	gint i;
	gchar *c;
	
	for (i = 0, c = new_text; c;
	     c = g_utf8_find_next_char (c, (new_text_length != -1) ?
	     	(new_text + new_text_length) : NULL)) {
		if (*c < '0' || *c > '9') {
			g_signal_stop_emission (entry, g_signal_lookup (
				"insert-text", GTK_TYPE_EDITABLE), 0);
			gdk_beep ();
			break;
		}
	}
}

static gboolean
dates_details_time_entry_alt (GtkArrow *source, const gchar *entry,
			      DatesViewData *d, gint limit)
{
	gboolean overflow = FALSE;
	GtkEntry *widget;
	gchar buffer[33];
	gint hour;
	GtkArrowType direction;
	
	widget = GTK_ENTRY (glade_xml_get_widget (d->xml, entry));
	gtk_widget_grab_focus (GTK_WIDGET (widget));
	hour = atoi (gtk_entry_get_text (widget));
	g_object_get (G_OBJECT (source), "arrow-type", &direction, NULL);
	if (direction == GTK_ARROW_UP)
		hour ++;
	else
		hour --;
	if (hour < 0) { hour = limit; overflow = TRUE; }
	if (hour > limit) { hour = 0; overflow = TRUE; }
	snprintf (buffer, 33, "%d", hour);
	gtk_entry_set_text (widget, buffer);
	
	return overflow;
}

static void
dates_details_time_hour_cb (GtkButton *source, DatesViewData *d)
{
	dates_details_time_entry_alt (GTK_ARROW (gtk_bin_get_child (
		GTK_BIN (source))), "hour_entry", d, 23);
}

static void
dates_details_time_lminute_cb (GtkButton *source, DatesViewData *d)
{
	dates_details_time_entry_alt (GTK_ARROW (gtk_bin_get_child (
		GTK_BIN (source))), "lminute_entry", d, 5);
}

static void
dates_details_time_rminute_cb (GtkButton *source, DatesViewData *d)
{
	if (dates_details_time_entry_alt (GTK_ARROW (gtk_bin_get_child (
	    GTK_BIN (source))), "rminute_entry", d, 9))
		dates_details_time_lminute_cb (source, d);
}
#if 0
static void
dates_repeats_cb (GtkWidget *source, DatesViewData *d)
{
	GtkWidget *widget;
	
	gtk_window_set_transient_for (
		GTK_WINDOW (glade_xml_get_widget (d->xml, "time_dialog")),
		GTK_WINDOW (glade_xml_get_widget (d->xml, "repeats_dialog")));

	/* Run dialog */
	widget = glade_xml_get_widget (d->xml, "repeats_dialog");
	gtk_dialog_run (GTK_DIALOG (widget));
	gtk_widget_hide (widget);

	gtk_window_set_transient_for (
		GTK_WINDOW (glade_xml_get_widget (d->xml, "time_dialog")),
		GTK_WINDOW (glade_xml_get_widget (d->xml, "details_dialog")));
}

static void
dates_details_repeats_cb (GtkComboBox *combobox, DatesViewData *d)
{
	gint option = gtk_combo_box_get_active (combobox);
	
	if (option == OTHER)
		dates_repeats_cb (GTK_WIDGET (combobox), d);
	else {
		/* Ask for time */
		gtk_widget_show (glade_xml_get_widget (
			d->xml, "time_forever_checkbutton"));
		gtk_widget_hide (glade_xml_get_widget (d->xml, "time_dialog"));
	}
}
#endif
static void
dates_details_time_cb (DatesViewData *d, ECalComponentDateTime *time)
{
	GtkWidget *widget;
#ifndef WITH_HILDON
	gint i, hour, minute;
	gchar buffer[33];
	GtkCalendar *calendar;
	
	calendar = GTK_CALENDAR (
		glade_xml_get_widget (d->xml, "time_calendar"));
	
	/* Set time widgets */	
	gtk_calendar_select_month (calendar, time->value->month - 1,
		time->value->year);
	gtk_calendar_select_day (calendar, time->value->day);

	g_snprintf (buffer, 33, "%d", time->value->hour);
	gtk_entry_set_text (GTK_ENTRY (
		glade_xml_get_widget (d->xml, "hour_entry")), buffer);
	i = time->value->minute / 10;
	g_snprintf (buffer, 33, "%d", i);
	gtk_entry_set_text (GTK_ENTRY (
		glade_xml_get_widget (d->xml, "lminute_entry")), buffer);
	i = time->value->minute - (i * 10);
	g_snprintf (buffer, 33, "%d", i);
	gtk_entry_set_text (GTK_ENTRY (
		glade_xml_get_widget (d->xml, "rminute_entry")), buffer);
#else
	HildonDateEditor *date_edit = HILDON_DATE_EDITOR (glade_xml_get_widget (
		d->xml, "time_date_editor"));
	HildonTimeEditor *time_edit = HILDON_TIME_EDITOR (glade_xml_get_widget (
		d->xml, "time_time_editor"));
	
	/* Set date */
	hildon_date_editor_set_year (date_edit, time->value->year);
	hildon_date_editor_set_month (date_edit, time->value->month);
	hildon_date_editor_set_day (date_edit, time->value->day);
	
	/* Set time */
	hildon_time_editor_set_time (time_edit, time->value->hour,
		time->value->minute, 0);
	hildon_time_editor_show_seconds (time_edit, FALSE);
#endif
	
	gtk_widget_hide (glade_xml_get_widget (
		d->xml, "time_forever_checkbutton"));
	gtk_widget_show (glade_xml_get_widget (d->xml, "time_table"));

	/* Run dialog */
	widget = glade_xml_get_widget (d->xml, "time_dialog");
	gtk_dialog_run (GTK_DIALOG (widget));
	gtk_widget_hide (widget);
	
	/* Retrieve time */
#ifndef WITH_HILDON
	gtk_calendar_get_date (calendar,
		(guint *)(&time->value->year),
		(guint *)(&time->value->month),
		(guint *)(&time->value->day));
	time->value->month += 1;
	
	hour = atoi (gtk_entry_get_text (GTK_ENTRY (
		glade_xml_get_widget (d->xml, "hour_entry"))));
	if ((hour >= 0) && (hour <= 23))
		time->value->hour = hour;
	minute = (atoi (gtk_entry_get_text (GTK_ENTRY (
		glade_xml_get_widget (d->xml, "lminute_entry")))) * 10) +
		atoi (gtk_entry_get_text (GTK_ENTRY (
		glade_xml_get_widget (d->xml, "rminute_entry"))));
	if ((minute >= 0) && (minute <= 59))
		time->value->minute = minute;
#else
	time->value->year = hildon_date_editor_get_year (date_edit);
	time->value->month = hildon_date_editor_get_month (date_edit);
	time->value->day = hildon_date_editor_get_day (date_edit);
	
	hildon_time_editor_get_time (time_edit,
		(guint *)(&time->value->hour),
		(guint *)(&time->value->minute),
		(guint *)(&time->value->second));
#endif
}

static void
dates_details_time_start_cb (GtkWidget *source, DatesViewData *d)
{
	ECalComponentDateTime time;
	struct tm timem;
	gchar time_text[256];
	
	e_cal_component_get_dtstart (d->comp, &time);
	dates_details_time_cb (d, &time);
	timem = icaltimetype_to_tm (time.value);
	strftime (time_text, 256, TIME_MARKUP, &timem);
	gtk_label_set_markup (GTK_LABEL (
		glade_xml_get_widget (d->xml, "details_start_label")),
		time_text);
	e_cal_component_set_dtstart (d->comp, &time);
	e_cal_component_free_datetime (&time);
	dates_view_redraw (d->view, FALSE);
}

static void
dates_details_time_end_cb (GtkWidget *source, DatesViewData *d)
{
	ECalComponentDateTime time;
	struct tm timem;
	gchar time_text[256];
	
	e_cal_component_get_dtend (d->comp, &time);
	dates_details_time_cb (d, &time);
	timem = icaltimetype_to_tm (time.value);
	strftime (time_text, 256, TIME_MARKUP, &timem);
	gtk_label_set_markup (GTK_LABEL (
		glade_xml_get_widget (d->xml, "details_end_label")),
		time_text);
	e_cal_component_set_dtend (d->comp, &time);
	e_cal_component_free_datetime (&time);
	dates_view_redraw (d->view, FALSE);
}

static void
dates_open_cb (GtkWidget *source, DatesViewData *d)
{
	if (d->read_only) return;
	/* TODO: Handle recurring events */
	if (e_cal_component_has_recurrences (d->comp)) return;
	dates_fill_details_dialog (d->view, d);
	gtk_widget_show (glade_xml_get_widget (d->xml, "details_dialog"));
}

static void
dates_details_ok_cb (GtkWidget *source, DatesViewData *d)
{
	ECalComponentText text;
	gchar *desc;
	GSList *desc_list;
	GtkTextIter start, end;
	GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (
		glade_xml_get_widget (d->xml, "details_textview")));
	GtkWidget *widget;
	ECalComponentText summary, new_summary;

	gtk_text_buffer_get_start_iter (buffer, &start);
	gtk_text_buffer_get_end_iter (buffer, &end);
	desc = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
	text.value = desc ? desc : "";
	text.altrep = NULL;
	desc_list = g_slist_prepend (NULL, &text);
	e_cal_component_set_description_list (d->comp, desc_list);
	if (desc) g_free (desc);
	g_slist_free (desc_list);
	
	e_cal_component_set_location (d->comp, gtk_entry_get_text (GTK_ENTRY (
		glade_xml_get_widget (d->xml, "details_location_entry"))));
	
	widget = glade_xml_get_widget (d->xml, "details_summary_entry");
	e_cal_component_get_summary (d->comp, &summary);
	new_summary.altrep = summary.altrep;
	new_summary.value = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
	e_cal_component_set_summary (d->comp, &new_summary);

	gtk_widget_hide (glade_xml_get_widget (d->xml, "details_dialog"));
	
	d->waiting = NONE;
	dates_commit_event_cb (NULL, d, CALOBJ_MOD_ALL);
}

static void
dates_delete_cb (GtkWidget *source, DatesViewData *d)
{
	ECalComponentText summary;
	GtkWidget *widget;
	const char *value;

	e_cal_component_get_summary (d->comp, &summary);
	widget = glade_xml_get_widget (d->xml, "details_dialog");
	if (GTK_WIDGET_VISIBLE (widget))
		value = gtk_entry_get_text (GTK_ENTRY (glade_xml_get_widget (
			d->xml, "details_summary_entry")));
	else if (summary.value)
		value = summary.value;
	else if (summary.altrep)
		value = summary.altrep;
	else {
		g_warning ("Deleting event with no summary");
		value = "Unknown event";
	}
		
	widget = gtk_message_dialog_new (
		GTK_WINDOW (glade_xml_get_widget (d->xml, "main_window")),
		GTK_DIALOG_MODAL,
		GTK_MESSAGE_QUESTION,
		GTK_BUTTONS_NONE,
		_("Are you sure you want to delete event '%s'?"),
		value);
	gtk_dialog_add_buttons (GTK_DIALOG (widget),
		_("No, do not delete"), GTK_RESPONSE_NO,
		_("Yes, delete this event"), GTK_RESPONSE_YES,
		NULL);
	
	if (gtk_dialog_run (GTK_DIALOG (widget)) == GTK_RESPONSE_YES) {
		const char *uid = NULL;
		e_cal_component_get_uid (d->comp, &uid);
		e_cal_remove_object (d->cal, uid, NULL);

		/* Hide the details dialog, in case we deleted from there */
		d->waiting = NONE;
		gtk_widget_hide (glade_xml_get_widget (
			d->xml, "details_dialog"));
	}
	
	gtk_widget_destroy (widget);
}

static void
dates_details_cancel_cb (GtkWidget *source, DatesViewData *d)
{
	GtkWidget *widget;
	
	widget = glade_xml_get_widget (d->xml, "details_dialog");
	gtk_widget_hide (widget);
	
	if (d->waiting == NEW_EVENT) {
		const char *uid = NULL;
		d->waiting = NONE;
		e_cal_component_get_uid (d->comp, &uid);
		e_cal_remove_object (d->cal, uid, NULL);
	}
}

static gboolean
dates_details_close_cb (GtkWidget *widget, GdkEvent *event, DatesViewData *d)
{
	dates_details_cancel_cb (widget, d);
	return TRUE;
}

static gboolean
dates_select_event_idle_cb (gpointer data)
{
	DatesViewData *d = data;
	gboolean returnval =
		!dates_view_set_selected_event (d->view, d->uri_uid, NULL);
	
	if (!returnval) {
		g_free (d->uri_uid);
#ifndef WITH_HILDON
		gtk_widget_hide (
			glade_xml_get_widget (d->xml, "details_message_hbox"));
#endif
		if (d->widgets) {
			contacts_set_widgets_sensitive (d->widgets);
			g_list_free (d->widgets);
			d->widgets = NULL;
		}
		if (d->waiting == NEW_EVENT)
			dates_fill_details_dialog (d->view, d);
		else
			d->waiting = NONE;
		gtk_widget_show (
			glade_xml_get_widget (d->xml, "details_dialog"));
	}
	
	return returnval;
}

static void
dates_snap_time_to_30 (struct icaltimetype *t)
{
	if (t->minute < 15) {
		t->minute = 0;
	} else if (t->minute < 45) {
		t->minute = 30;
	} else {
		t->minute = 0;
		icaltime_adjust (t, 0, 1, 0, 0);
	}
}

static void
dates_new_cb (GtkWidget *source, DatesViewData *d)
{
	GtkTreeIter iter;
	icalcomponent *comp;
	char *uid = NULL;
	struct icalperiodtype period;
	ECal *c;
	GError *error = NULL;

	g_message ("Entered new_cb");
    
	/* FIXME: Use default calendar, not first */
	if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (d->cal_list_store),
	    &iter)) {
		g_warning ("gtk_tree_model_get_iter_first failed");
		return;
	}

	if (!dates_view_get_selected_period (d->view, &period)) {
		struct icaltimetype date, now;

		/* Create a default time event */
		date = *(dates_view_get_date (d->view));
		now = icaltime_current_time_with_zone (date.zone);
		date.is_date = FALSE;
		now.is_date = FALSE;
		period.duration = icaldurationtype_null_duration ();
		period.duration.minutes = 0;
		/* If we're looking at today, create an event that starts in
		 * a couple of hours - otherwise create an event starting at
		 * 9am. If we're in month/year view, create all-day events.
		 * TODO: Make the default time configurable/read Evo's setting
		 */
		if ((icaltime_compare_date_only (date, now) == 0) &&
		    (d->zoom < 11)) {
			period.duration.hours = 2;
			period.start = icaltime_add (now, period.duration);
			if (icaltime_compare_date_only (
			    date, period.start) != 0) {
				/* If we're now on a different date, shunt the
				 * time to 9am again.
				 */
				period.start.hour = 9;
				period.start.minute = 0;
			}
		} else {
			/* NOTE: 11 is the magic number where we're viewing a 
			 * month or more. See dates_zoom_change.
			 */
			if (d->zoom >= 11) {
				period.start = date;
			} else {
				period.duration.hours = 9;
				period.start = icaltime_add (
					date, period.duration);
			}
		}
		
		dates_snap_time_to_30 (&period.start);
		
		period.duration.hours = (d->zoom >= 11) ? 24 : 1;
		period.end = icaltime_add (period.start, period.duration);
	}
	
	comp = icalcomponent_new_vevent ();
	icalcomponent_set_dtstart (comp, period.start);
	icalcomponent_set_dtend (comp, period.end);
	icalcomponent_set_summary (comp, _("New event"));
	gtk_tree_model_get (GTK_TREE_MODEL (d->cal_list_store), &iter,
		COL_CALPTR, &c, -1);
	d->waiting = NEW_EVENT;
	if (e_cal_create_object (c, comp, (char **)&uid, &error)) {
		d->uri_uid = g_strconcat (e_cal_get_uri (c), uid, NULL);
		g_idle_add (dates_select_event_idle_cb, d);
	} else {
		g_warning ("Failed to create calendar object: %s",
			error->message);
		g_error_free (error);
	}
}

static void
dates_calendar_combo_changed_cb (GtkComboBox *widget, DatesViewData *d)
{
	ECal *c;
	ECalComponent *comp = d->comp;
	GtkTreeIter iter;
	
	gtk_combo_box_get_active_iter (widget, &iter);
	gtk_tree_model_get (gtk_combo_box_get_model (widget),
		&iter, COL_CALPTR, &c, -1);
	
	if (c != d->cal) {
		/* Move event between calendars */
		const char *uid = NULL;
		
		g_object_ref (comp);
		if (!d->waiting)
			d->waiting = CAL_CHANGE;
		e_cal_component_get_uid (comp, &uid);
		e_cal_remove_object (d->cal, uid, NULL);
		
		/* TODO: Error checking */
		e_cal_create_object (c,
			e_cal_component_get_icalcomponent (comp),
			(char **)&uid, NULL);
		g_object_unref (comp);
		d->uri_uid = g_strconcat (e_cal_get_uri (c), uid, NULL);
		d->widgets = contacts_set_widgets_desensitive (
			glade_xml_get_widget (d->xml, "details_dialog"));
		/* TODO: Show the saving note when the save takes > 0.5s? */
/*		gtk_widget_show (glade_xml_get_widget (
			d->xml, "details_message_hbox"));*/
		g_idle_add (dates_select_event_idle_cb, d);
	}
}

/* The following function taken from gnome clock-applet
 * (gnome-panel/applets/clock/calendar-sources.c)
 */
static ECal *
load_esource (ESource        *esource,
	      ECalSourceType  source_type,
	      GSList         *existing_clients)
{
  ECal   *retval;
  GError *error;


  if (existing_clients)
    {
      GSList *l;

      for (l = existing_clients; l; l = l->next)
	{
	  ECal *client = E_CAL (l->data);

	  if (e_source_equal (esource, e_cal_get_source (client)))
	    {
	      g_debug ("load_esource: found existing source, returning that");

	      return g_object_ref (client);
	    }
	}
    }

  retval = e_cal_new (esource, source_type);
  if (!retval)
    {
      g_warning ("Could not load source '%s' from '%s'\n",
		 e_source_peek_name (esource),
		 e_source_get_uri (esource));
      return NULL;
    }

  error = NULL;
  if (!e_cal_open (retval, TRUE, &error))
    {
      g_assert (error != NULL);
      g_warning ("Cannot open calendar from uri '%s': %s\n",
		 e_cal_get_uri (retval), error->message);
      g_error_free (error);
      g_object_unref (retval);
      return NULL;
    }

  g_debug ("Loaded calendar from uri '%s'",
	   e_cal_get_uri (retval));

  return retval;
}

static void
dates_backend_died_cb (ECal *client, DatesViewData *d)
{
	g_debug ("Calendar died, removing");
	dates_view_remove_calendar (d->view, client);
}

static void
dates_sources_changed_cb (ESourceList *cal_list, DatesViewData *d)
{
	g_message ("Sources changed, reloading calendars");
	dates_update_calendars (cal_list, d);
}

static void
dates_cal_open_cb (ECal *ecal, ECalendarStatus status, DatesViewData *d)
{
	gboolean read_only;
	GError *error = NULL;
	const char *cal_name;
	ESource *source;
	GtkTreeIter iter;

	g_debug ("Calendar opened, setting up...");
	
	if (status != E_CALENDAR_STATUS_OK) {
		g_warning ("Error '%d' opening ecal", status);
		g_object_unref (ecal);
		if (d->first_load) {
			GtkWidget *dialog = gtk_message_dialog_new (
				GTK_WINDOW (glade_xml_get_widget (
					d->xml, "main_window")),
				GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
				GTK_MESSAGE_ERROR,
				GTK_BUTTONS_CLOSE,
				"Dates failed to open the system calendar. "
				"You will not be able to view or create "
				"events; this may be remedied by restarting.");
			gtk_dialog_run (GTK_DIALOG (dialog));
		}
		return;
	}
	
	source = e_cal_get_source (ecal);

	/* Create the system calendar when it isn't found */
	if (d->first_load) {
		gchar *group_uri;
		ESourceGroup *group;
		ESourceList *cal_list;
		const gchar *cal_uri = e_cal_get_uri (ecal);
		gint size;
		GError *error = NULL;
		
		g_debug ("First time run, creating new system calendar");
		g_debug ("New system calendar URI: %s", cal_uri);
		
		/* FIXME: This whole section is a little hacky, do it better */
		
		/* Create group by chopping off last bit of system calendar uri
		 */
		size = g_strrstr (cal_uri, "/system") - cal_uri;
		group_uri = g_new (gchar, size);
		strncpy (group_uri, cal_uri, size + 1);
		group_uri[size] = '\0';
		g_debug ("Creating new group at %s", group_uri);
		group = e_source_group_new ("On This Computer", group_uri);
		g_free (group_uri);
		
		/* Name system calendar and add it to group */
		e_source_set_absolute_uri (source, cal_uri);
		/* NOTE: Default Evolution colour */
		e_source_set_color (source, 0xBECEDD);
		e_source_set_name (source, "Personal");
		e_source_group_add_source (group, source, 0);
		
		/* Add group to default list */
		cal_list = e_source_list_new_for_gconf_default (
			CALENDAR_GCONF_SOURCES);
		e_source_list_add_group (cal_list, group, 0);
		if (!e_source_list_sync (cal_list, &error)) {
			g_warning ("Error syncing ESourceList: %s",
				error->message);
			g_error_free (error);
			return;
		}
		
		g_signal_connect (G_OBJECT (cal_list), "changed",
			G_CALLBACK (dates_sources_changed_cb), d);
		d->first_load = FALSE;
		
		/* Return, the sources will change immediately after this */
		g_debug ("Created new system calendar and group");
		return;
	}

	cal_name = e_source_peek_name (source);
	
	if (!e_cal_is_read_only (ecal, &read_only, &error)) {
		g_warning ("Error querying calendar's read-only status: %s",
			error->message);
		g_error_free (error);
		g_object_unref (ecal);
		return;
	}
			
	gtk_list_store_insert_with_values (
		d->cal_list_store, &iter, G_MAXINT,
		COL_CALNAME, cal_name ? cal_name : "Unknown",
		COL_CALPTR, ecal,
		COL_NOTREADONLY, !read_only, -1);
	dates_view_add_calendar (d->view, ecal);
		
	g_signal_connect (G_OBJECT (ecal), "backend_died",
		G_CALLBACK (dates_backend_died_cb), d);

	g_debug ("Calendar added.");
}

static void
dates_update_calendars (ESourceList *cal_list, DatesViewData *d)
{
	GSList *groups, *g;

	/* TODO: Don't reload *all* calendars on update */
	gtk_list_store_clear (d->cal_list_store);
	dates_view_remove_all_calendars (d->view);
	groups = e_source_list_peek_groups (cal_list);

	g_message ("Updating Calendars");
    
	for (g = groups; g; g = g->next) {
		GSList *sources, *s;
		g_debug ("Group %s (%s)",
			e_source_group_peek_name (E_SOURCE_GROUP (g->data)),
			e_source_group_peek_base_uri (
				E_SOURCE_GROUP (g->data)));
		sources = e_source_group_peek_sources (
			E_SOURCE_GROUP (g->data));
		for (s = sources; s; s = s->next) {
			ESource *source = E_SOURCE (s->data);
			ECal *ecal;
			
			ecal = load_esource (source,
				E_CAL_SOURCE_TYPE_EVENT, NULL);
			if (!ecal) {
				g_message ("No EVENTS in this source");
				continue;
			}
            
			g_message ("Found EVENTS in this source");

			g_signal_connect (G_OBJECT (ecal), "cal_opened",
				G_CALLBACK (dates_cal_open_cb), d);
			/* FIXME: Check if we're leaking a reference by not
			 * unreffing here.
			 */
			e_cal_open_async (ecal, TRUE);
		}
	}	
}

static gboolean
dates_load_calendars (DatesViewData *d)
{
	ESourceList *cal_list;

	/* Load calendars */
	cal_list = e_source_list_new_for_gconf_default (CALENDAR_GCONF_SOURCES);

	if (!cal_list) {
		g_error ("Error loading ESource list");
		return FALSE;
	}

	if (!e_source_list_peek_groups (cal_list)) {
		g_warning ("No groups found");
		g_object_unref (cal_list);
		return FALSE;
	}

	g_signal_connect (G_OBJECT (cal_list), "changed",
		G_CALLBACK (dates_sources_changed_cb), d);
	dates_update_calendars (cal_list, d);
	
	return TRUE;
}

#ifdef WITH_HILDON
GtkWidget *
glade_hildon_window_new (GladeXML *xml, GType widget_type,
			 GladeWidgetInfo *info)
{
	return hildon_window_new ();
}

GtkWidget *
glade_hildon_date_editor_new (GladeXML *xml, GType widget_type,
			      GladeWidgetInfo *info)
{
	return hildon_date_editor_new ();
}

GtkWidget *
glade_hildon_time_editor_new (GladeXML *xml, GType widget_type,
			      GladeWidgetInfo *info)
{
	return hildon_time_editor_new ();
}

static gboolean
dates_key_press_cb (GtkWidget *widget, GdkEventKey *event, DatesViewData *d)
{
	GtkWidget *window = glade_xml_get_widget (d->xml, "main_window");
	GtkWidget *header = glade_xml_get_widget (d->xml, "header_eventbox");
	switch (event->keyval) {
		case GDK_F6 :	/* Full-screen key */
			if (!(gdk_window_get_state (window->window) &
			    GDK_WINDOW_STATE_FULLSCREEN)) {
				gtk_window_fullscreen (GTK_WINDOW (widget));
				gtk_widget_show (header);
			} else {
				gtk_window_unfullscreen (GTK_WINDOW (widget));
				gtk_widget_hide (header);
			}
			return TRUE;
		case GDK_F8 :	/* Zoom out key */
			dates_zoom_out_cb (NULL, d);
			return TRUE;
		case GDK_F7 :	/* Zoom in key */
			dates_zoom_in_cb (NULL, d);
			return TRUE;
	}
	
	return FALSE;
}
#endif

int
main (int argc, char **argv)
{
	GtkWidget *widget;
	GladeXML *xml;
	DatesViewData data;
	GtkCellRenderer *renderer;
	GtkTreeModel *filter;
	GConfClient *client;
	GConfBridge *bridge;

#ifdef WITH_HILDON
	HildonProgram *program;
	osso_context_t *osso_context;
#endif
	
	/* Standard initialisation for gtk and glade */
	gtk_init (&argc, &argv);
	glade_init ();

#ifdef WITH_HILDON
	g_message ("Creating HildonProgram");
	g_set_application_name (_("Dates"));
	program = hildon_program_new ();

	g_message ("Initialising HildonProgram");
	osso_context = osso_initialize ("dates", "0.1", TRUE, NULL);

	if (!osso_context) {
		g_critical("Could not initialize libosso\n");
		return 1;
	}

	glade_register_widget (HILDON_TYPE_WINDOW,
		glade_hildon_window_new,
		glade_standard_build_children,
		NULL);
	glade_register_widget (HILDON_TYPE_DATE_EDITOR,
		glade_hildon_date_editor_new,
		NULL, NULL);
	glade_register_widget (HILDON_TYPE_TIME_EDITOR,
		glade_hildon_time_editor_new,
		NULL, NULL);
#endif
	
	/* Initialise the i18n support code */
	bindtextdomain (GETTEXT_PACKAGE, DATES_LOCALE_DIR);;
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	/* Set critical errors to close application */
	g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL);

	/* Load up main_window from interface xml, quit if xml file not found */
	xml = glade_xml_new (XML_FILE, NULL, NULL);
	if (!xml)
		g_critical ("Could not find interface XML file '%s'",
			    XML_FILE);

#ifdef WITH_HILDON
	hildon_program_set_common_menu(program,
		GTK_MENU(glade_xml_get_widget (xml, "main_menubar")));
	hildon_program_set_common_toolbar (program,
		GTK_TOOLBAR (glade_xml_get_widget (xml, "common_toolbar"))); 
#endif
	
	/* Hook up signals defined in interface xml */
/*	glade_xml_signal_autoconnect (xml);*/

	widget = glade_xml_get_widget (xml, "dates_view");
	g_assert(widget);
	
#ifdef WITH_HILDON
	dates_view_set_use_dragbox (DATES_VIEW (widget), FALSE);
#endif
	client = gconf_client_get_default ();
	data.view = DATES_VIEW (widget);
	data.zoom = gconf_client_get_int (client, DATES_GCONF_ZOOM, NULL);
	if (data.zoom == 0) {
		data.zoom = 3;
		gtk_widget_set_sensitive (glade_xml_get_widget (xml,
			"zoom_in_button"), FALSE);
	} else if (data.zoom == 3) {
		gtk_widget_set_sensitive (glade_xml_get_widget (xml,
			"zoom_in_button"), FALSE);
	} else if (data.zoom == 16) {
		gtk_widget_set_sensitive (glade_xml_get_widget (xml,
			"zoom_out_button"), FALSE);
	}
	data.xml = xml;
	data.comp = NULL;
	data.widgets = NULL;
	data.waiting = NONE;
#ifndef WITH_HILDON
	/* Perhaps in a future version? */
/*	dates_view_set_use_list (data.view, TRUE);*/
#endif
	dates_zoom_change (data.zoom, data.view);

	/* TODO: UI for multiple calendars */
	/* Load calendars */
	data.cal_list_store = gtk_list_store_new (COL_LAST, G_TYPE_STRING,
		G_TYPE_POINTER, G_TYPE_BOOLEAN);
	data.first_load = FALSE;
	if (!dates_load_calendars (&data)) {
		/* Ensure system calendar exists */
		data.first_load = TRUE;
		ECal *ecal = e_cal_new_system_calendar ();

		g_debug ("Trying to create/open system calendar...");
		g_assert (ecal);

		g_signal_connect (G_OBJECT (ecal), "cal_opened",
				  G_CALLBACK (dates_cal_open_cb), &data);

		e_cal_open_async (ecal, FALSE);
	}
	
	/* Set transient parent for dialogs */
	gtk_window_set_transient_for (
		GTK_WINDOW (glade_xml_get_widget (xml, "details_dialog")),
		GTK_WINDOW (glade_xml_get_widget (xml, "main_window")));
	gtk_window_set_transient_for (
		GTK_WINDOW (glade_xml_get_widget (xml, "time_dialog")),
		GTK_WINDOW (glade_xml_get_widget (xml, "details_dialog")));
	gtk_window_set_transient_for (
		GTK_WINDOW (glade_xml_get_widget (xml, "exceptions_dialog")),
		GTK_WINDOW (glade_xml_get_widget (xml, "details_dialog")));
	
#ifdef WITH_HILDON
	/* HildonTime/DateEditor are broken, show_all them */
	gtk_widget_show_all (glade_xml_get_widget (xml, "time_date_editor"));
	gtk_widget_show_all (glade_xml_get_widget (xml, "time_time_editor"));
#endif
	
	/* Create tree model filter to filter out read-only calendars */
	filter = gtk_tree_model_filter_new (
		GTK_TREE_MODEL (data.cal_list_store), NULL);
	gtk_tree_model_filter_set_visible_column (
		GTK_TREE_MODEL_FILTER (filter), COL_NOTREADONLY);
	
	/* Set tree model for details dialog calendar selector */
	widget = glade_xml_get_widget (xml, "details_calendar_combobox");
	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), filter);
	renderer = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, TRUE);
	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (widget),
		renderer, "text", COL_CALNAME);
	
	/* Set nice colours for full-screen date header */
	gtk_widget_set_state (glade_xml_get_widget (xml, "header_eventbox"),
		GTK_STATE_SELECTED);
	gtk_widget_set_state (glade_xml_get_widget (xml, "header_label"),
		GTK_STATE_SELECTED);

	/* Bind widgets to gconf properties */
	bridge = gconf_bridge_get ();
	gconf_bridge_bind_window (bridge, DATES_GCONF_PREFIX, GTK_WINDOW (
		glade_xml_get_widget (xml, "main_window")), TRUE, TRUE);
	gconf_bridge_bind_property (bridge, DATES_GCONF_PREFIX,
		G_OBJECT (data.view), "week_start");

	/* Connect signals */
	
	/* DatesView signals */
	widget = glade_xml_get_widget (xml, "dates_view");
	/* Set date label */
	dates_date_changed_cb (DATES_VIEW (widget), &data);
	g_signal_connect (G_OBJECT (widget), "date_changed",
			  G_CALLBACK (dates_date_changed_cb), &data);
	g_signal_connect (G_OBJECT (widget), "event_selected",
			  G_CALLBACK (dates_event_selected_cb), &data);
	g_signal_connect (G_OBJECT (widget), "event_moved",
			  G_CALLBACK (dates_event_moved_cb), &data);
	g_signal_connect (G_OBJECT (widget), "event_sized",
			  G_CALLBACK (dates_event_sized_cb), &data);
/*	g_signal_connect (G_OBJECT (widget), "commit_event",
			  G_CALLBACK (dates_commit_event_cb), &data);*/
	g_signal_connect (G_OBJECT (widget), "event_activated",
			  G_CALLBACK (dates_open_cb), &data);
	
	/* Main window signals */
	widget = glade_xml_get_widget (xml, "main_window");
	g_signal_connect (G_OBJECT (widget), "destroy",
			  G_CALLBACK (gtk_main_quit), NULL);
	widget = glade_xml_get_widget (xml, "zoom_out_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_zoom_out_cb), &data);
	widget = glade_xml_get_widget (xml, "zoom_in_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_zoom_in_cb), &data);
/*	widget = glade_xml_get_widget (xml, "day_button");
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (widget), FALSE);
	g_signal_connect (G_OBJECT (widget), "toggled",
			  G_CALLBACK (dates_day_cb), &data);
	widget = glade_xml_get_widget (xml, "week_button");
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (widget), FALSE);
	g_signal_connect (G_OBJECT (widget), "toggled",
			  G_CALLBACK (dates_week_cb), &data);
	widget = glade_xml_get_widget (xml, "month_button");
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (widget), FALSE);
	g_signal_connect (G_OBJECT (widget), "toggled",
			  G_CALLBACK (dates_month_cb), &data);
	widget = glade_xml_get_widget (xml, "year_button");
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (widget), FALSE);
	g_signal_connect (G_OBJECT (widget), "toggled",
			  G_CALLBACK (dates_year_cb), &data);*/
	widget = glade_xml_get_widget (xml, "back_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_back_cb), &data);
	widget = glade_xml_get_widget (xml, "back_menuitem");
	g_signal_connect (G_OBJECT (widget), "activate",
			  G_CALLBACK (dates_back_cb), &data);

	widget = glade_xml_get_widget (xml, "forward_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_forward_cb), &data);
	widget = glade_xml_get_widget (xml, "forward_menuitem");
	g_signal_connect (G_OBJECT (widget), "activate",
			  G_CALLBACK (dates_forward_cb), &data);

	widget = glade_xml_get_widget (xml, "today_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_today_cb), &data);
	
	widget = glade_xml_get_widget (xml, "open_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_open_cb), &data);
	widget = glade_xml_get_widget (xml, "calendar_open_menuitem");
	g_signal_connect (G_OBJECT (widget), "activate",
			  G_CALLBACK (dates_open_cb), &data);

	widget = glade_xml_get_widget (xml, "delete_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_delete_cb), &data);
	widget = glade_xml_get_widget (xml, "calendar_delete_menuitem");
	g_signal_connect (G_OBJECT (widget), "activate",
			  G_CALLBACK (dates_delete_cb), &data);

	widget = glade_xml_get_widget (xml, "new_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_new_cb), &data);
	widget = glade_xml_get_widget (xml, "calendar_new_menuitem");
	g_signal_connect (G_OBJECT (widget), "activate",
			  G_CALLBACK (dates_new_cb), &data);

	widget = glade_xml_get_widget (xml, "about_menuitem");
	g_signal_connect (G_OBJECT (widget), "activate",
			  G_CALLBACK (dates_about_cb), &data);
    
	/* Details dialog signals */
	widget = glade_xml_get_widget (xml, "details_dialog");
	g_signal_connect (G_OBJECT (widget), "delete_event",
			  G_CALLBACK (dates_details_close_cb), &data);
	widget = glade_xml_get_widget (xml, "details_ok_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_ok_cb), &data);
	widget = glade_xml_get_widget (xml, "details_cancel_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_cancel_cb), &data);
	widget = glade_xml_get_widget (xml, "details_delete_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_delete_cb), &data);
	widget = glade_xml_get_widget (xml, "details_start_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_time_start_cb), &data);
	widget = glade_xml_get_widget (xml, "details_end_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_time_end_cb), &data);
	widget = glade_xml_get_widget (xml, "details_calendar_combobox");
	g_signal_connect (G_OBJECT (widget), "changed",
			  G_CALLBACK (dates_calendar_combo_changed_cb), &data);
/*	widget = glade_xml_get_widget (xml, "details_repeats_combobox");
	g_signal_connect (G_OBJECT (widget), "changed",
			  G_CALLBACK (dates_details_repeats_cb), &data);*/
	
	/* Time dialog signals */
#ifndef WITH_HILDON
	widget = glade_xml_get_widget (xml, "hour_up_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_time_hour_cb), &data);
	widget = glade_xml_get_widget (xml, "hour_down_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_time_hour_cb), &data);
	widget = glade_xml_get_widget (xml, "lminute_up_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_time_lminute_cb), &data);
	widget = glade_xml_get_widget (xml, "lminute_down_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_time_lminute_cb), &data);
	widget = glade_xml_get_widget (xml, "rminute_up_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_time_rminute_cb), &data);
	widget = glade_xml_get_widget (xml, "rminute_down_button");
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (dates_details_time_rminute_cb), &data);
	widget = glade_xml_get_widget (xml, "hour_entry");
	g_signal_connect (G_OBJECT (widget), "insert-text",
			  G_CALLBACK (dates_details_time_entry_changed), &data);
	widget = glade_xml_get_widget (xml, "lminute_entry");
	g_signal_connect (G_OBJECT (widget), "insert-text",
			  G_CALLBACK (dates_details_time_entry_changed), &data);
	widget = glade_xml_get_widget (xml, "rminute_entry");
	g_signal_connect (G_OBJECT (widget), "insert-text",
			  G_CALLBACK (dates_details_time_entry_changed), &data);
#endif

	widget = glade_xml_get_widget (xml, "main_window");
	gtk_widget_show (widget);
#ifdef WITH_HILDON
	hildon_program_add_window (program, HILDON_WINDOW (widget));
	g_signal_connect (G_OBJECT (widget), "key_press_event",
			  G_CALLBACK (dates_key_press_cb), &data);
#endif

	/* Exceptions dialog signals */

	gtk_main ();
	
	return 0;
}
